/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function CommonFilter(sId, required)
{
	if (sId)
	{
		this.init(sId, required);
	}
}
CommonFilter.prototype.init = function(sId, required)
{
	this.sId = sId;
	this.required = required;
	this.parameter = '';
	this.adapter = null;
	this.adapterInitialized = false;
	this.showWarnHandles = new Array();
	this.hideWarnHandles = new Array();
}
CommonFilter.prototype.setSubmitParameter = function(parameter)
{
	this.parameter = parameter;
}
CommonFilter.prototype.getSubmitParam = function()
{
	return this.parameter;
}
CommonFilter.prototype.getId = function()
{
	return this.sId;
}
CommonFilter.prototype.setManager = function(manager)
{
	this.manager = manager;
}
CommonFilter.prototype.setSelectionModel = function(model)
{
	this.model = model;
}
CommonFilter.prototype.setFilterValueAdapter = function(adapter)
{
	this.adapter = adapter;
}
CommonFilter.prototype.registerShowWarnHandle = function(action)
{
	this.showWarnHandles.push(action);
}
CommonFilter.prototype.registerHideWarnHandle = function(action)
{
	this.hideWarnHandles.push(action);
}
CommonFilter.prototype.start = function()
{
	if (this.adapterInitialized == false)
		this.initAdapter();
}
CommonFilter.prototype.initAdapter = function()
{
	if (this.adapter == null)
		this.adapter = new FilterValueAdapter();
	this.adapter.setModel(this.model);
	this.adapterInitialized = true;
}
CommonFilter.prototype.selectionModelChanged = function(model)
{
	if (this.manager != null && this.manager.isAutoCommit())
		this.manager.process();
}
CommonFilter.prototype.getCommandValues = function()
{
	if (this.adapterInitialized == false)
		this.initAdapter();
	return this.adapter.getCommandValues();
}
CommonFilter.prototype.isValid = function()
{
	if (!this.required)
		return true;
	return this.model.hasSelectedValues();
}
CommonFilter.prototype.hasSelectedValue = function()
{
	return this.model.hasSelectedValues();
}
CommonFilter.prototype.validate = function()
{
	var valid = this.isValid();
	if (valid)
		this.fireHideWarn();
	else
		this.fireShowWarn();
	return valid;
}
CommonFilter.prototype.fireHideWarn = function()
{
	for (var i = 0; i < this.hideWarnHandles.length; i++)
		this.hideWarnHandles[i].invoke();
}
CommonFilter.prototype.fireShowWarn = function()
{
	for (var i = 0; i < this.showWarnHandles.length; i++)
		this.showWarnHandles[i].invoke();
}
CommonFilter.prototype.reset = function()
{
	this.model.reset();
	this.fireHideWarn();
}
CommonFilter.prototype.isActive = function()
{
	return true;
}
function FilterValueAdapter()
{
	this.init();
}
FilterValueAdapter.prototype.init = function()
{
	this.model = null;
}
FilterValueAdapter.prototype.setModel = function(model)
{
	this.model = model;
}
FilterValueAdapter.prototype.getCommandValues = function()
{
	if (this.model.hasSelectedValues() == false)
		return new Array();
	return this.model.getActiveValues();
}
FilterValueAdapterBothOpen.prototype = new FilterValueAdapter();
FilterValueAdapterBothOpen.prototype.constructor = FilterValueAdapterBothOpen;
FilterValueAdapterBothOpen.superclass = FilterValueAdapter.prototype;
function FilterValueAdapterBothOpen()
{
	this.init();
}
FilterValueAdapterBothOpen.prototype.init = function()
{
	FilterValueAdapterBothOpen.superclass.init.call(this);
}
FilterValueAdapterBothOpen.prototype.getCommandValues = function()
{
	var cmdValues = new Array();
	var values = this.model.getActiveValues();
	if (values == null || values.length == 0)
		return cmdValues;
	
	var firstIndex = this.model.getIndexOf(values[0]);
	var lastIndex = this.model.getIndexOf(values[1]);
	if (firstIndex == 0 && lastIndex == this.model.getNumberOfValues() - 1)
	{
		
		return cmdValues;
	}
	
	lastIndex = lastIndex - 1;
	for (var i = firstIndex; i < lastIndex + 1; i ++)
		cmdValues.push(this.model.getSelectionValueByIndex(i))
	return cmdValues;
}
FilterValueAdapterLeftOpen.prototype = new FilterValueAdapter();
FilterValueAdapterLeftOpen.prototype.constructor = FilterValueAdapterLeftOpen;
FilterValueAdapterLeftOpen.superclass = FilterValueAdapter.prototype;
function FilterValueAdapterLeftOpen()
{
	this.init();
}
FilterValueAdapterLeftOpen.prototype.init = function()
{
	FilterValueAdapterLeftOpen.superclass.init.call(this);
}
FilterValueAdapterLeftOpen.prototype.getCommandValues = function()
{
	var cmdValues = new Array();
	var values = this.model.getActiveValues();
	if (values == null || values.length == 0)
		return cmdValues;
	
	var firstIndex = this.model.getIndexOf(values[0]);
	var lastIndex = this.model.getNumberOfValues() - 1;
	if (firstIndex == 0)
	{
		
		return cmdValues;
	}
	for (var i = firstIndex; i < lastIndex + 1; i ++)
		cmdValues.push(this.model.getSelectionValueByIndex(i))
	return cmdValues;
}
FilterValueAdapterRightOpen.prototype = new FilterValueAdapter();
FilterValueAdapterRightOpen.prototype.constructor = FilterValueAdapterRightOpen;
FilterValueAdapterRightOpen.superclass = FilterValueAdapter.prototype;
function FilterValueAdapterRightOpen()
{
	this.init();
}
FilterValueAdapterRightOpen.prototype.init = function()
{
	FilterValueAdapterRightOpen.superclass.init.call(this);
}
FilterValueAdapterRightOpen.prototype.getCommandValues = function()
{
	var cmdValues = new Array();
	var values = this.model.getActiveValues();
	if (values == null || values.length == 0)
		return cmdValues;
	
	var firstIndex = 0;
	var lastIndex = this.model.getIndexOf(values[0]);
	if (lastIndex == this.model.getNumberOfValues() - 1)
	{
		
		return cmdValues;
	}
	for (var i = firstIndex; i < lastIndex + 1; i ++)
		cmdValues.push(this.model.getSelectionValueByIndex(i))
	return cmdValues;
}
