/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function Evaluation( sId, question, answer, bCorrect, bSolution )
{
  if( sId )
  {
    this.init(sId, question, answer, bCorrect, bSolution);
  }
}
Evaluation.prototype.init = function( sId, question, answer, bCorrect, bSolution )
{
  ;
  this.sId = sId;
  this.workItem = null;
  this.bActive = false;
  this.bCorrect = bCorrect;
  this.bSolution = bCorrect;
  if( bSolution == false )
  {
    this.bSolution = false;
  } 
  this.question = question;
  this.answer = answer;
  this.sOrigin = '';
  this.bUserActivated = false;
  this.bUserDeactivated = false;
  this.bSystemConstructive = false;
  this.bSystemCorrective = false;
  this.bSystemPartialConstructive = false;
  this.bSystemPartialCorrective = false;
  this.bChangeForDisabledOnReset = true;
  this.bUncovered = false;
  this.bEvaluated = false;
  this.aSCListener = new Array();
  this.nSCListener = 0;
  this.bIsInitialActive = false;
}
Evaluation.prototype.getId = function()
{
  return this.sId;
}
Evaluation.prototype.toString = function()
{
  var s = '' +
          ' q=' + this.question.getId() +
          ' a=' + this.answer.getId() +
          ' active=' + (this.bActive   ? '1'  : '0') +
          ' correct=' + (this.bCorrect  ? '1' : '0') +
          ' solution=' + (this.bSolution ? '1': '0') +
          ' id=' + this.sId +
          ' ';
  return s;
}
Evaluation.prototype.setSilentlyActive = function( bActive, sOrigin )
{
  var notifyFlagBackup = this.workItem.getNotifyManagers();
  this.workItem.setNotifyManagers(false);
  this.setActive(bActive, sOrigin);
  this.workItem.setNotifyManagers(notifyFlagBackup);
}
Evaluation.prototype.setActive = function( bActive, sOrigin )
{
  ;
  if( this.canSetActive(bActive) )
  {
    this.workItem.willChange(this);
    this.sOrigin = (sOrigin == undefined) ? 'user' : sOrigin;
    if( bActive )
    {
      switch( this.sOrigin )
              {
        case 'user':
          this.bUserActivated = true;
          break;
        case 'system_brand':
          this.bSystemCorrective = true;
          break;
        case 'system_off':
          break;
        case 'system_setpartial':
          break;
        default:
          this.bSystemConstructive = true;
      }
    }
    else
    {
      switch( this.sOrigin )
              {
        case 'user':
          this.bUserDeactivated = true;
          break;
        case 'system_off':
        case 'system_swapsingle':
          break;
        default:
          this.bSystemCorrective = true;
      }
    }
    this.bActive = bActive;
    this.question.changedEvaluation(this);
    this.answer.changedEvaluation(this);
    this.notify();
  }
}
Evaluation.prototype.canSetActive = function( bActive )
{
  var bcanSetActive = true;
  if( this.isActive() != bActive ) 
  {
    bcanSetActive = this.answer.canSetActive(bActive) && this.question.canSetActive(bActive);
  }
  return bcanSetActive;
}
Evaluation.prototype.hasUserContribution = function()
{
  return this.bUserActivated || this.bUserDeactivated;
}
Evaluation.prototype.hasSystemContribution = function()
{
  return this.bSystemConstructive || this.bSystemCorrective;
}
Evaluation.prototype.hasSystemConstuction = function()
{
  return this.bSystemConstructive;
}
Evaluation.prototype.hasSystemCorrection = function()
{
  return this.bSystemCorrective;
}
Evaluation.prototype.setSystemConstuction = function( bSysCon )
{
  this.bSystemConstructive = bSysCon;
}
Evaluation.prototype.setChangeForDisabledOnReset = function( bChangeForDisabledOnReset )
{
  this.bChangeForDisabledOnReset = bChangeForDisabledOnReset;
}
Evaluation.prototype.setSystemCorrection = function( bSysCor )
{
  this.bSystemCorrective = bSysCor;
}
Evaluation.prototype.hasSystemPartialConstuction = function()
{
  return this.bSystemPartialConstructive;
}
Evaluation.prototype.hasSystemPartialCorrection = function()
{
  return this.bSystemPartialCorrective;
}
Evaluation.prototype.setSystemPartialConstuction = function( bSysCon )
{
  this.bSystemPartialConstructive = bSysCon;
}
Evaluation.prototype.setSystemPartialCorrection = function( bSysCor )
{
  this.bSystemPartialCorrective = bSysCor;
}
Evaluation.prototype.isActive = function()
{
  return this.bActive;
}
Evaluation.prototype.isCorrect = function()
{
  return this.bCorrect;
}
Evaluation.prototype.isSolution = function()
{
  return this.bSolution;
}
Evaluation.prototype.isSolved = function()
{
  //;
  var bSolved = undefined;
  if( this.bActive )
  {
    bSolved = this.bCorrect;
  }
  else
  {
    if( this.workItem.bWithAlternatives )
    {
      if( this.bCorrect && this.question.nMax > 1 )
      {
        bSolved = false;
      }
    }
    else
    {
      bSolved = this.question.isSolved() && this.answer.isSolved();
    }
  }
  return bSolved;
}
Evaluation.prototype.isEvaluated = function()
{
  return this.bEvaluated;
}
Evaluation.prototype.getOrigin = function()
{
  return this.sOrigin;
}
Evaluation.prototype.getQuestion = function()
{
  return this.question;
}
Evaluation.prototype.getAnswer = function()
{
  return this.answer;
}
Evaluation.prototype.registerStatusChangeListener = function( scListener )
{
  ;
  if( this.aSCListener[scListener.getId()] != null )
  {
    ;
  }
  else
  {
    this.aSCListener[scListener.getId()] = scListener;
    this.nSCListener++;
    ;
    scListener.setModelItem(this);
  }
  return null;
}
Evaluation.prototype.setWorkItem = function( workItem )
{
  ;
  this.workItem = workItem;
}
Evaluation.prototype.start = function()
{
  ;
}
Evaluation.prototype.evaluate = function()
{
  ;
  if( this.workItem.bWithAlternatives )
  {
    if( this.bActive )
    {
      if( !this.bCorrect )
      {
        this.answer.evaluated(false);
        this.question.evaluated(false);
      }
      else
      {
        if( this.answer.nMax == 1 )
        {
          this.answer.evaluated(true);
        }
        if( this.question.nMax == 1 )
        {
          this.question.evaluated(true);
        }
      }
    }
  }
  else
  {
    this.answer.evaluate();
    this.question.evaluate();
  }
  this.bEvaluated = true;
  this.notify();
}
Evaluation.prototype.notify = function()
{
  ;
  if( this.workItem.getNotifyView() )
  {
    for( var id in this.aSCListener )
    {
      this.aSCListener[id].changed(this);
    }
    this.workItem.changed(this);
  }
}
Evaluation.prototype.isUncovered = function()
{
  //;
  return this.bUncovered;
}
Evaluation.prototype.setUncovered = function( bUncovered )
{
  //;
  this.bUncovered = bUncovered;
}
Evaluation.prototype.checkUncovered = function()
{
  ;
  if( this.hasSystemConstuction() )
  {
    this.setUncovered(true);
    if( this.answer.getMax() == 1 )
    {
      this.answer.uncoverEvaluations();
    }
    if( this.question.getMax() == 1 )
    {
      this.question.uncoverEvaluations();
    }
  }
  if( this.hasSystemCorrection() )
  {
    this.setUncovered(true);
  }
  return this.isUncovered();
}
Evaluation.prototype.hasOffItem = function()
{
  ;
  return this.answer.isOffItem() || this.question.isOffItem();
}
Evaluation.prototype.hasUserFocus = function()
{
  ;
  return this.answer.hasUserFocus() || this.question.hasUserFocus();
  ;
}
Evaluation.prototype.getEvaluation = function()
{
  //;
  return this;
}
Evaluation.prototype.countActiveEvaluations = function()
{
  //;
  return this.isActive() ? 1 : 0;
}
Evaluation.prototype.reset = function()
{
  ;
  this.bUserActivated = false;
  this.bUserDeactivated = false;
  this.bSystemConstructive = false;
  this.bSystemCorrective = false;
  this.bSystemPartialConstructive = false;
  this.bSystemPartialCorrective = false;
  this.bUncovered = false;
  this.bEvaluated = false;
  if( this.bChangeForDisabledOnReset )
  {
    this.setActive(false, 'user');
  }
  else
  {
    if( this.isActive() )
    {
      this.setActive(false, 'user');
    }
  }
}
